@@ -73,13 +73,15 @@ sub requires {
 	'File::Spec'	=> 0,
 	'HTTP::Tiny'	=> 0,
 	'IPC::Cmd'	=> 0,
-	'Pod::Perldoc'	=> 0,
+	'Module::Load::Conditional'	=> 0,
+	'Pod::Perldoc'			=> 0,
 	'Pod::Simple::LinkSection'	=> 0,
 	'Pod::Simple::SimpleTree'	=> 0,
 	'Scalar::Util'	=> 0,
 	'Storable'	=> 0,
 	'Test::Builder'	=> 0,
 	strict		=> 0,
+	utf8		=> 0,
 	warnings	=> 0,
 	@extra,
     };
